/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Tracy Norris (tnorris@nc.rr.com), Jesper Kristensen (jesper@nc.rr.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
 // This holds the object for accessing the preferences
var autofillPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService).getBranch("autofill.");

setInternalPrefs();

// catch page loads
window.addEventListener('load', setButtonStatus, true);

// catch tab events
document.getElementById("content").addEventListener ("select", setButtonStatus, true); 

// preference observer
var prefChangeObserver = {
  observe :

  function(aSubject, aTopic, aData) {
	 setButtonStatus();
  }
};

// add observer to preference
var qi = autofillPrefs.QueryInterface(Components.interfaces["nsIPrefBranchInternal"]);
qi.addObserver("profile_list", prefChangeObserver, false);


// Set Button Status based on configuration
function setButtonStatus() {
	consoleDump("In setButtonStatus");

	// call doVisualize to turn the button on/off based on page content
	doVisualize();
	
	var autofillButton = document.getElementById("autofill-button");
	if (autofillPrefs.prefHasUserValue("profile_list")) {
		var profileList=autofillPrefs.getCharPref("profile_list");

		// if there are no profiles, disable the button (regardless of page content)
		if (profileList.length==0) {
			consoleDump("Disabling Attribute (no profiles)");
			autofillButton.setAttribute("disabled",true);
			autofillButton.removeAttribute("context");
		}
		// if there is only one profile, keep the status of the button, but make it not a menu button (and disable context)
		else if (profileList.indexOf(",")==-1) {
			consoleDump("UnMenuing Button");
			autofillButton.removeAttribute("type");
			autofillButton.removeAttribute("context");
		}
		// If there is more than one profile, leave the status of the button, but make it a menu button
		else {
			consoleDump("More than one profile, making menu-button");
			autofillButton.setAttribute("type","menu-button");
		}
	}
	else {
		consoleDump("Disabling Attribute (no config)");
		autofillButton.setAttribute("disabled",true);
		autofillButton.removeAttribute("type");
		autofillButton.removeAttribute("context");
	}
}

function populateAutofillMenu(event) {
	if (autofillPrefs.prefHasUserValue("selected_profile_id"))
		selectedProfileId=autofillPrefs.getCharPref("selected_profile_id");
	else
		selectedProfileId="";
		
	
	clearMenu(event.target);
	
	if (autofillPrefs.prefHasUserValue("profile_list"))
		profileList=autofillPrefs.getCharPref("profile_list");
	else
		profileList="";

	var profiles=profileList.split(",");

	// Only show the list of profiles if there is more than 1
	if (profiles.length>1) {
		for (i=0;i<profiles.length;++i) {
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label",profiles[i]+" "); // Add a space at the end to prevent theme conflicts (like profiles named "home")
			menuitem.setAttribute("index",i);
			menuitem.setAttribute("id",profiles[i]);
			menuitem.setAttribute("type","radio");
			menuitem.setAttribute("name","profiles");

			if (profiles[i]==selectedProfileId) {
				consoleDump("Checking: " + profiles[i]);
				menuitem.setAttribute("checked",true);
			}

			event.target.appendChild(menuitem);
		}

		consoleDump("Profiles length: " + profiles.length);
	}

	consoleDump("onPopulate: " + event.target.id + ":" + event.target.getAttribute("showOptions"));
	
	if (event.target.getAttribute("showOptions")=="true") { // ToDo: is there a way to return a custom attribute as a boolean?
		if (profiles.length<=1)
			optionsIndex=0;
		else
			optionsIndex=profiles.length+1;

		if (optionsIndex>0) {
			var menuSeparator=document.createElement("menuseparator");
			menuSeparator.setAttribute("index",profiles.length);
			event.target.appendChild(menuSeparator);
		}

		var menuitem = document.createElement("menuitem");
		menuitem.setAttribute("label","Autofill Options");
		menuitem.setAttribute("index",optionsIndex);
		menuitem.setAttribute("id","autofill-menu-options");
		menuitem.setAttribute("oncommand","showOptions();event.preventBubble();");
		event.target.appendChild(menuitem);
	}
	
	return true;
}

function showOptions() {
	window.openDialog("chrome://autofill/content/autofill_pref_dialog.xul","autofill_pref_dialog","chrome,centerscreen");
}


function doProfilePick(event) {
	consoleDump("Pick Profile:" + event.target.id);
	var selectedProfileId=autofillPrefs.setCharPref("selected_profile_id", event.target.id);
	return true;
}


function clearMenu(aParent) {
  var children = aParent.childNodes;
  for (var i = 0; i < children.length; i++) {
      var index = children[i].getAttribute("index");
      if (index)
        aParent.removeChild(children[i]);
   }
}


function doAutofill(event) {
	processFields(true);
}


function flattenText(node) {
	
	var flText = "";
	if (node.firstChild != null) {
		for (var c=0;c<node.childNodes.length && node.childNodes[c].nodeName.toLowerCase() != "input";c++) {
			var nodeindex = (node.childNodes.length - c) - 1; // do it backwards!
			flText = flText + flattenText(node.childNodes[c]);
		}
	}
	else {
		if (node.nodeValue != null) {
			flText = node.nodeValue;
		}
	}
	return flText;
}

function getLabelForNode(node) {	
	
	var prevNode = node;
	var nodeText = "";
	for(var backnodes=0;backnodes<5 && prevNode!=null;backnodes++) {
				
		if (prevNode.previousSibling != null) {
			prevNode = prevNode.previousSibling;
			nodeText = flattenText(prevNode) + nodeText;
		}
		else if (prevNode.parentNode != null) {
			prevNode = prevNode.parentNode;
			if (prevNode.nodeValue != null) {
				nodeText = prevNode.nodeValue;
			}
		}
		else {
			prevNode = null;
		}		
		
		if (prevNode.nodeName.toLowerCase() == 'input') {
			prevNode = null;
		}
	}		
	
	return nodeText;
}

function trim(s) {
	// Remove leading spaces and carriage returns
	while ((s.substring(0,1) == ' ') || (s.substring(0,1) == '\n') || (s.substring(0,1) == '\r')) {
		s = s.substring(1,s.length);
	}

	// Remove trailing spaces and carriage returns
	while ((s.substring(s.length-1,s.length) == ' ') || (s.substring(s.length-1,s.length) == '\n') || (s.substring(s.length-1,s.length) == '\r')) {
		s = s.substring(0,s.length-1);
	}
	
	return s;
}


function doVisualize() {
	
	if (autofillPrefs.prefHasUserValue("profile_list")) {			
		processFields(false);			
		if (autofillPrefs.getBoolPref("autohighlight") == true) {
			var linkElm = window.content.document.createElement("link");
			linkElm.setAttribute("id", "autofillVisualize");
			linkElm.setAttribute("href", "chrome://autofill/skin/visualize.css");
			linkElm.setAttribute("rel", "stylesheet");
			linkElm.setAttribute("type", "text/css");
			
			window.content.document.documentElement.childNodes[0].appendChild(linkElm);		
		}
	}
	else {
		document.getElementById("autofill-button").setAttribute("disabled","true");		
	}
}


function processFields(bChange) {

	var autofillPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("autofill.");
	var formFields = new Array("full_name","title","first_name","middle_name","last_name","suffix","phone_number","primary_line1",
				"primary_line2","primary_city","primary_state","primary_zip","primary_country","secondary_line1","secondary_line2",
				"card_name","card_type","card_number","card_verification","card_exp_day","card_exp_month","card_exp_year",
				"secondary_city","secondary_state","secondary_zip","secondary_country","primary_country_code","secondary_country_code","email_address");								
	var encryptedCCFields = "card_name,card_type,card_number,card_verification,card_exp_day,card_exp_month,card_exp_year";
	var profiles=autofillPrefs.getCharPref("profile_list");	
	var enableAutofill = false;	
	var password = "";
	var populateCCFields = false;
	
	if (profiles.length>0) {
		
		
		if (bChange) {
			var metaElements = window.content.document.getElementsByTagName("meta");
			for (var me = 0; me < metaElements.length; me++) {
				if (metaElements[me].name == "autofill" && metaElements[me].content == "encrypted") {							
					if (autofillPrefs.prefHasUserValue("password")) {
						if (checkPassword()) {
							populateCCFields = true;
							createAutofillKey();
						}
					}					
					me = metaElements.length;
				}
			}
		}
		
		var profileList=profiles.split(",");

		// select correct profile
		var profilenumber=0;
		for (var i=0;i<profileList.length;i++) {
			if (autofillPrefs.getCharPref("selected_profile_id") == profileList[i]) {
				profilenumber = i;
			}
		}

		var formValues = {};
		for (var i=1;i<formFields.length;i++) {
			formValues[formFields[i]]=autofillPrefs.getCharPref(profileList[profilenumber]+"."+formFields[i]);
		}
		
		// Copy primary address values to secondary if required
		if (autofillPrefs.getBoolPref(profileList[profilenumber]+".use_primary") == true) {
			formValues["secondary_city"] = formValues["primary_city"];
			formValues["secondary_country"] = formValues["primary_country"];
			formValues["secondary_country_code"] = formValues["primary_country_code"];
			formValues["secondary_line1"] = formValues["primary_line1"];
			formValues["secondary_line2"] = formValues["primary_line2"];
			formValues["secondary_state"] = formValues["primary_state"];
			formValues["secondary_zip"] = formValues["primary_zip"];
		}
		
		// Setup fullname
		formValues["full_name"] = formValues["first_name"];
		if (formValues["middle_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["middle_name"];
		}
		if (formValues["last_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["last_name"];
		}
		if (formValues["suffix"] != "") {
			formValues["full_name"] = formValues["full_name"] + ", " + formValues["suffix"];
		}
		//formFields[formFields.length] = "full_name";  //already included in the array above

		// get all form fields
		var formElements = new Array();		
		
		var inputElements = window.content.document.getElementsByTagName("input");
		for (var j = 0; j < inputElements.length; j++) {
			formElements[formElements.length] = inputElements[j];
		}
		var selectElements = window.content.document.getElementsByTagName("select");
		for (var j = 0; j < selectElements.length; j++) {
			formElements[formElements.length] = selectElements[j];
		}
		

		var encryptedField = false;

		// loop through all input fields
		for (var j = 0; j < formElements.length; j++)	{
			// get attributes
			var elmType = formElements[j].type.toLowerCase();
			var elmName = formElements[j].name.toLowerCase();
			             			
			if (elmType == "input" || elmType == "text" || elmType == "textarea" || elmType == "password" || elmType == "select-one" || elmType == "select-multiple") {	
												
				var match = false;
				for (var l=0;l<formFields.length;l++) {					
					var fieldnamelist = autofillPrefs.getCharPref("form_name." + formFields[l]);
					var fieldnames = fieldnamelist.split(",");	
					if (fieldnames.length > 0) {
						for (var k=0;k<fieldnames.length;k++) {
							if (elmName == fieldnames[k].toLowerCase()) {
								match = true;								
								if (bChange) {
									if (encryptedCCFields.indexOf(formFields[l]) != -1) {
										if (populateCCFields) {			
											var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),autofillKey,"CBC"));											
											
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	decrypted;										
											}											
										}
									}	
									else {
										if (elmType == "select-one" || elmType == "select-multiple") {
											var selElement = formElements[j];
											for (var c=0; c<selElement.length;c++ ) {
												if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
													selElement[c].selected = true;
												}
											}			                        
										
										}
										else {
											formElements[j].value =	formValues[formFields[l]];										
										}
									}
															
								}								
								else {
									formElements[j].setAttribute("autofillsupport","true");
									enableAutofill = true;
									if (encryptedCCFields.indexOf(formFields[l]) != -1) {
										encryptedField = true;
									}								
								}
							}
						}
					}			
				}	
				if (!match) {  // try field substring matching
					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = autofillPrefs.getCharPref("form_label_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (elmName.indexOf(fieldnames[k].toLowerCase()) != -1) {
									match = true;
									if (bChange) {
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											if (populateCCFields) {			
												var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),autofillKey,"CBC"));											

												if (elmType == "select-one" || elmType == "select-multiple") {
													var selElement = formElements[j];
													for (var c=0; c<selElement.length;c++ ) {
														if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
															selElement[c].selected = true;
														}
													}			                        

												}
												else {
													formElements[j].value =	decrypted;										
												}											
											}
										}	
										else {
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	formValues[formFields[l]];										
											}
										}

									}								
									else {
										formElements[j].setAttribute("autofillsupport","true");
										enableAutofill = true;
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											encryptedField = true;
										}								
									}
								}
							}
						}			
					}
				}
				if (!match) {  // try label matching
					consoleDump("No field name match for: " + elmName);
					var labelText = getLabelForNode(formElements[j]).toLowerCase();
					if (labelText.length > 50) {
						labelText = labelText.substring(labelText.length-50,labelText.length);
					}
					consoleDump("Trying with following label: " + labelText);						

					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = autofillPrefs.getCharPref("form_label_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (labelText.indexOf(fieldnames[k].toLowerCase()) != -1) {
									match = true;
									if (bChange) {
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											if (populateCCFields) {			
												var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),autofillKey,"CBC"));											

												if (elmType == "select-one" || elmType == "select-multiple") {
													var selElement = formElements[j];
													for (var c=0; c<selElement.length;c++ ) {
														if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
															selElement[c].selected = true;
														}
													}			                        

												}
												else {
													formElements[j].value =	decrypted;										
												}											
											}
										}	
										else {
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	formValues[formFields[l]];										
											}
										}

									}								
									else {
										formElements[j].setAttribute("autofillsupport","true");
										enableAutofill = true;
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											encryptedField = true;
										}								
									}
								}
							}
						}			
					}
	
				}																	
			}
		}
		if (encryptedField ) {
			var metaElm = window.content.document.createElement("meta");
			metaElm.setAttribute("name", "autofill");
			metaElm.setAttribute("content", "encrypted");			
			window.content.document.documentElement.childNodes[0].appendChild(metaElm);		
		}
	}
	
	if (!bChange) {
		if (enableAutofill) {
			document.getElementById('autofill-button').removeAttribute("disabled");			
			document.getElementById('autofill-button').setAttribute("context","autofill-menu");			
		}
		else {
			document.getElementById('autofill-button').setAttribute("disabled","true");	
			document.getElementById('autofill-button').removeAttribute("context");	
		}
	}
}


